///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoAttrCond.cc
// ---------------
// Cego database table field implementation
//     
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2000-2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: CegoAttrCond
// 
// Description: 
//
// Status: QG-2.5.5
//
///////////////////////////////////////////////////////////////////////////////

#include "CegoAttrCond.h"

CegoAttrCond::CegoAttrCond()
{
    _condMode = UNDEF;
}

CegoAttrCond::CegoAttrCond(const CegoAttrCond& p)
{
    _condMode = UNDEF;
}

CegoAttrCond::CegoAttrCond(const Chain attrName, CegoComparison comp, const CegoFieldValue& fv)
{
    _attrName = attrName;
    _comp = comp;
    _fv = fv;
    _condMode = VAL;
}

CegoAttrCond::CegoAttrCond(const Chain attrName, CegoComparison comp, const CegoAttrDesc& attrDesc)
{
    _attrName = attrName;
    _comp = comp;
    _attrDesc = attrDesc;
    _condMode = ATTR;
}

CegoAttrCond::CegoAttrCond(const Chain attrName, const CegoFieldValue& fv, const CegoFieldValue& fv2)
{
    _attrName = attrName;
    _fv = fv;
    _fv2 = fv2;
    _condMode = BTWN;
}


CegoAttrCond::~CegoAttrCond()
{
}

CegoAttrCond::CondMode CegoAttrCond::getCondMode() const
{
    return _condMode;
}

const Chain& CegoAttrCond::getAttrName() const
{
    return _attrName;
}

CegoComparison CegoAttrCond::getComparison() const
{
    return _comp;
}

void CegoAttrCond::setComparison(CegoComparison comp)
{
    _comp = comp;
}

const CegoFieldValue& CegoAttrCond::getFieldValue() const
{
    return _fv;
}

const CegoFieldValue& CegoAttrCond::getFieldValue2() const
{
    return _fv2;
}

const CegoAttrDesc& CegoAttrCond::getAttrDesc() const
{
    return _attrDesc;
}

bool CegoAttrCond::setup(const ListT<CegoField>& fl)
{
    bool isSet=false;
    CegoField* pF = fl.First();
    while (pF && isSet == false)
    {	
	if ( ( _attrDesc.getTableName() == pF->getTableName() || _attrDesc.getTableName()== pF->getTableAlias() )
	     && _attrDesc.getAttrName() == pF->getAttrName() )
	{
	    isSet=true;
	    _fv = pF->getValue();
	}
	pF = fl.Next();
    }
    return isSet;
}

bool CegoAttrCond::setup(ListT<CegoField>* joinBuf, int offset, int size)
{
    int i=offset;
    bool isSet=false;
    while ( i < size+offset && isSet == false )
    {
	CegoField* pF = joinBuf[i].First();
	while (pF && isSet == false)
	{	
	    // cout << "Checking T=" << pF->getTableName() << " A=" << pF->getAttrName() << "V=" << pF->getValue() << endl;
	    if ( ( _attrDesc.getTableName() == pF->getTableName() || _attrDesc.getTableName()== pF->getTableAlias() )
		 && _attrDesc.getAttrName() == pF->getAttrName() )
	    {
		isSet=true;
		
		_fv = pF->getValue();
	    }
	    pF = joinBuf[i].Next();
	}
	i++;
    }
    return isSet;
}

CegoAttrCond& CegoAttrCond::operator = ( const CegoAttrCond& ac)
{
    _attrName = ac._attrName;
    _comp = ac._comp;
    _fv = ac._fv;
    _fv2 = ac._fv2;
    _attrDesc = ac._attrDesc;
    _condMode = ac._condMode;
    return (*this);
}
    
bool CegoAttrCond::operator == ( const CegoAttrCond& ac)
{
    if ( _condMode == VAL )
	return (_attrName == ac._attrName && _comp == ac._comp && _fv == ac._fv);
    else
	return (_attrName == ac._attrName && _comp == ac._comp && _attrDesc == ac._attrDesc);
}

Chain CegoAttrCond::toChain() const
{
    Chain s;

    s = _attrName;

    if ( _condMode == BTWN )
    {
	s += Chain(" between ") + _fv.toChain() + Chain(" and ") + _fv2.toChain();
    }
    else
    {
	switch ( _comp )
	{
	case EQUAL:
	    s += Chain(" = ");
	    break;
	case NOT_EQUAL:
	    s += Chain(" != ");
	    break;
	case LESS_THAN:
	    s += Chain(" < ");
	    break;
	case MORE_THAN:
	    s += Chain(" > ");
	    break;
	case LESS_EQUAL_THAN:
	    s += Chain(" <= ");
	    break;
	case MORE_EQUAL_THAN:
	    s += Chain(" >= ");
	    break;
	}
	if ( _condMode == VAL )
	    s += _fv.toChain();
	else
	    s += _attrDesc.toChain(); //  + Chain("(") + _fv.toChain() + Chain(")");
    }
    return s;

}

ostream& operator << (ostream& s, const CegoAttrCond& ac)
{
    s << ac.toChain();
    return s;
}

    

