#ifndef _CEGOATTRCOND_H_INCLUDED_
#define _CEGOATTRCOND_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoAttrCond.h
// --------------
// Cego attribute condition structure class definition
//     
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: CegoAttrCond
// 
// Description: Attribute condition description used for index cursors
//
// Status: QG-2.5.5
//
///////////////////////////////////////////////////////////////////////////////

#include "ListT.h"
#include "Chain.h"

#include "CegoComparison.h"
#include "CegoField.h"
#include "CegoFieldValue.h"
#include "CegoAttrDesc.h"
#include "CegoContentObject.h"


class CondDesc;

class CegoAttrCond {
    
 public:

    enum CondMode { UNDEF, VAL, ATTR, BTWN };

    CegoAttrCond();
    CegoAttrCond(const CegoAttrCond& p);
    CegoAttrCond(const Chain attrName, CegoComparison comp, const CegoFieldValue& fv);
    CegoAttrCond(const Chain attrName, CegoComparison comp, const CegoAttrDesc& attrDesc);
    CegoAttrCond(const Chain attrName, const CegoFieldValue& fv, const CegoFieldValue& fv2);
    ~CegoAttrCond();

    CondMode getCondMode() const;

    const Chain& getAttrName() const;
    CegoComparison getComparison() const;
    void setComparison(CegoComparison comp);

    const CegoFieldValue& getFieldValue() const;
    const CegoFieldValue& getFieldValue2() const;
    const CegoAttrDesc& getAttrDesc() const;

    bool setup(const ListT<CegoField>& fl);
    bool setup(ListT<CegoField>* joinBuf, int offset, int size);

    CegoAttrCond& operator = ( const CegoAttrCond& ac);
    bool operator == ( const CegoAttrCond& ac);

    Chain toChain() const;
    friend ostream& operator << (ostream& s, const CegoAttrCond& ac);
      
 private:

    Chain _attrName;
    CegoComparison _comp;
    CegoFieldValue _fv;
    CegoFieldValue _fv2;
    CegoAttrDesc _attrDesc;
    CondMode _condMode;
};

#endif




