///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoMediatorConnection.cc
// -------------------------
// Cego mediator connection class implementation
//      
// Design and Implementation by Bjoern Lemke
//     
// (C)opyright 2000-2007 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: CegoMediatorConnection
// 
// Description: 
//
// Status: CLEAN
//
///////////////////////////////////////////////////////////////////////////////



#include "CegoMediatorConnection.h"
#include "Net.h"

#include "Host.h"
#include "CegoXMLdef.h"


CegoMediatorConnection::CegoMediatorConnection()
{
    _pN = 0;
    _pAdminHandle = 0;
}

CegoMediatorConnection::CegoMediatorConnection(const Chain& hostName)
{
    _hostName = hostName;
    _pN = 0;
    _pAdminHandle = 0;
}

CegoMediatorConnection::CegoMediatorConnection(const Chain& hostName, int portNo, const Chain& user, const Chain& passwd)
{
    _hostName = hostName;
    _portNo = portNo;
    _user = user;
    _passwd = passwd;
    _pN = 0;
    _pAdminHandle = 0;
}
 
CegoMediatorConnection::~CegoMediatorConnection()
{
    
    if ( _pN )
	delete _pN;

    if ( _pAdminHandle )
	delete _pAdminHandle;
}


void CegoMediatorConnection::setLogger(Logger *pLog)
{
    _pLog = pLog;
}

void CegoMediatorConnection::setHostName(const Chain& hostName)
{
    _hostName = hostName;
}

const Chain& CegoMediatorConnection::getHostName() const
{
    return _hostName;
}

void CegoMediatorConnection::setPortNo(int portNo)
{
    _portNo = portNo;
}

int CegoMediatorConnection::getPortNo() const
{
    return _portNo;
}

const Chain& CegoMediatorConnection::getUser() const
{
    return _user;
}

const Chain& CegoMediatorConnection::getPasswd() const
{
    return _passwd;
}

void CegoMediatorConnection::beat()
{   
    if ( _pAdminHandle )
    {
	Host h;
	_pAdminHandle->medNotifyMediator(h.getName(), XML_ONLINE_VALUE);
    }
}

void CegoMediatorConnection::connect()
{
    Net n ( NETMNG_MSG_BUFLEN );   
    _pN = n.connect(_hostName, _portNo);
    _pAdminHandle = new CegoAdminHandler(_pLog, _pN);
    _pAdminHandle->requestSession(_user, _passwd);
}

void CegoMediatorConnection::disconnect()
{
    
    if ( _pAdminHandle )
	_pAdminHandle->closeSession();

}

CegoMediatorConnection& CegoMediatorConnection::operator = ( const CegoMediatorConnection& mc )
{
    _hostName = mc._hostName;
    _portNo = mc._portNo;
    _user = mc._user;
    _passwd = mc._passwd;

    return (*this);
}

bool CegoMediatorConnection::operator == ( const CegoMediatorConnection& mc)
{    
    if (_hostName == mc._hostName)
	return true;
    return false;
}
	
ostream& operator << (ostream& s, const CegoMediatorConnection& mc)
{
    s <<  mc._hostName << ":" << mc._portNo << ":" << mc._user << ":" << mc._passwd;
    return s;
}


